#pragma once

#include <vector>
#include "ZeilenFormat.h"

#define SPALTEN 23
static inline long AsciiHexToLong( const char* p )
{
   long ret=0;
   if( !p )
      return ret;
   if( !*p )
      return ret;
   do
   {
      if( isxdigit(*p) )
         ret <<= 4;
      if(*p >= '0' && *p<= '9')
         ret += *p - '0';
      if(*p >= 'a' && *p<= 'f')
         ret += *p - 'a' + 10;
      if(*p >= 'A' && *p<= 'F')
         ret += *p - 'A' + 10;
      p++;
   }while( *p );
   return ret;
}
static inline long AsciiHexToByte( const char* p )
{
   long ret=0;

   if( !p )
      return ret;
   if( !*p )
      return ret;
   if(*p >= '0' && *p<= '9')
      ret = *p - '0';
   if(*p >= 'a' && *p<= 'f')
      ret = *p - 'a' + 10;
   if(*p >= 'A' && *p<= 'F')
      ret = *p - 'A' + 10;
   p++;
   if( !*p )
      return ret;
   if(*p >= '0' && *p<= '9')
      ret = (ret<<4) + *p - '0';
   if(*p >= 'a' && *p<= 'f')
      ret = (ret<<4) + *p - 'a' + 10;
   if(*p >= 'A' && *p<= 'F')
      ret = (ret<<4) + *p - 'A' + 10;
   return ret;
}
#define ENDIAN32(val)( ((val) & 0xFF000000)>>24  \
   | ((val) & 0x00FF0000)>>8   \
   | ((val) & 0x0000FF00)<<8   \
   | ((val) & 0x000000FF)<<24)
#define ENDIAN16(val)( ((val) & 0x00FF)<<8   \
   | ((val) & 0xFF00)>>8)

#define CHANGE_ENDIAN16(val) (val = ENDIAN16( val ))
#define CHANGE_ENDIAN32(val) (val = ENDIAN32( val ))

template<class T_>
inline void ChangeEndianT(T_& val)
{
   switch( sizeof (val) )
   {
   case 1:
      return;
   case 2:
      val = ENDIAN16(val);
      return;
   case 4:
      val = ENDIAN32(val);
      return;
   case 8:
      {
         __int64 tmp = *(__int64*)(&val);

         tmp = ((tmp) & 0xFF00000000000000i64) >> 56
            |  ((tmp) & 0x00FF000000000000i64) >> 40  
            |  ((tmp) & 0x0000FF0000000000i64) >> 24  
            |  ((tmp) & 0x000000FF00000000i64) >> 8  
            |  ((tmp) & 0x00000000FF000000i64) << 8  
            |  ((tmp) & 0x0000000000FF0000i64) << 24  
            |  ((tmp) & 0x000000000000FF00i64) << 40  
            |  ((tmp) & 0x00000000000000FFi64) << 56  
            ;
         *(__int64*)(&val) = tmp;
      }
      return;
   }
}
template<>
inline void ChangeEndianT(LPVOID& /*val*/){}  //Pointer drehen verhindern

template<class T_>
inline T_ EndianT(const T_& val)
{
   T_ t(val);
   ChangeEndianT(t);
   return t;
}




struct CVarZeileIf
{
   virtual void GetData( BYTE& pData, int& nMaxSize ){}
};

class CVarZeile
{
public:
   CVarZeile(void);
   virtual ~CVarZeile(void);

   void CreateControls(CWnd* pDlg, CPoint ptStart, UINT nIDStart, UINT nIDDelta);
   void SetDataPointer( BYTE* p )
   {
      m_pData = p;
   }
   void SetDataAddress( WORD addr )
   {
      m_wAdd = addr;
   }
   void SetDataFormat( CZeilenFormat* pFormat )
   {
      m_pFormat = pFormat;
   }

   void OnBnClicked( int nSpalte );

   void DisableControls()   ;
   void EnableControls ()   ;
   void UpdateControls( int nExclude=-1);
   void OnEnChange( int nSpalte );
   static void CreateTopics(CWnd* pDlg, CPoint ptStart, UINT nIDStart, std::vector<CStatic*>& arTopics );
   CVarZeileIf* m_pVarZeileIf;
   BYTE*   m_pData          ;
   CZeilenFormat* m_pFormat ;

   UINT    m_nID            ;
   WORD    m_wAdd           ;
   int     m_nAs            ;
   BYTE    m_As__08         ;
   WORD    m_As__16         ;
   DWORD   m_As__32         ;
   char    m_As_S08         ;
   short   m_As_S16         ;
   long    m_As_S32         ;
   float   m_As__fl         ;
   double  m_As__db         ;
   BOOL    m_bSined         ;
   BOOL    m_bBigEndian     ;

   static const int  SpalteName      =  0 ;
   static const int  SpalteNeu       =  1 ;
   static const int  SpalteDel       =  2 ;
   static const int  SpalteAddr      =  3 ;
   static const int  Spalte_08       =  4 ;
   static const int  Spalte_16       =  5 ;
   static const int  Spalte_32       =  6 ;
   static const int  Spalte_fl       =  7 ;
   static const int  Spalte_db       =  8 ;
   static const int  SpalteSigned    =  9 ;
   static const int  SpalteBigEndian =  10 ;
   static const int  SpalteChar      =  11 ;
   static const int  SpalteHex8      =  12 ;
   static const int  SpalteDirekt    =  13 ;
   static const int  SpalteHex       =  14 ;
   static const int  SpalteAs__08    =  15 ;
   static const int  SpalteAs__16    =  16 ;
   static const int  SpalteAs__32    =  17 ;
   static const int  SpalteAs_S08    =  18 ;
   static const int  SpalteAs_S16    =  19 ;
   static const int  SpalteAs_S32    =  20 ;
   static const int  SpalteAs__fl    =  21 ;
   static const int  SpalteAs__db    =  22 ;
                                        

   CEdit   m_editName       ;
   CButton m_btNeu          ;
   CButton m_btDel          ;
   CEdit   m_editAddr       ;
   CButton m_bt_08          ;
   CButton m_bt_16          ;
   CButton m_bt_32          ;
   CButton m_bt_fl          ;
   CButton m_bt_db          ;
   CButton m_btSigned       ;
   CButton m_btBigEndian    ;
   CEdit   m_editChar       ;
   CEdit   m_editHex8       ;
   CEdit   m_editDirekt     ;
   CEdit   m_editHex        ;
   CEdit   m_editAs__08     ;
   CEdit   m_editAs__16     ;
   CEdit   m_editAs__32     ;
   CEdit   m_editAs_S08     ;
   CEdit   m_editAs_S16     ;
   CEdit   m_editAs_S32     ;
   CEdit   m_editAs__fl     ;
   CEdit   m_editAs__db     ;

   static int m_nBreite_editName       ;
   static int m_nBreite_btNeu          ;
   static int m_nBreite_btDel          ;
   static int m_nBreite_editAddr       ;
   static int m_nBreite_bt_08          ;
   static int m_nBreite_bt_16          ;
   static int m_nBreite_bt_32          ;
   static int m_nBreite_bt_fl          ;
   static int m_nBreite_bt_db          ;
   static int m_nBreite_btSigned       ;
   static int m_nBreite_btBigEndian    ;
   static int m_nBreite_editChar       ;
   static int m_nBreite_editHex8       ;
   static int m_nBreite_editDirekt     ;
   static int m_nBreite_editHex        ;
   static int m_nBreite_editAs__08     ;
   static int m_nBreite_editAs__16     ;
   static int m_nBreite_editAs__32     ;
   static int m_nBreite_editAs_S08     ;
   static int m_nBreite_editAs_S16     ;
   static int m_nBreite_editAs_S32     ;
   static int m_nBreite_editAs__fl     ;
   static int m_nBreite_editAs__db     ;

   static CString m_strTopic_editName       ;
   static CString m_strTopic_btNeu          ;
   static CString m_strTopic_btDel          ;
   static CString m_strTopic_editAddr       ;
   static CString m_strTopic_bt_08          ;
   static CString m_strTopic_bt_16          ;
   static CString m_strTopic_bt_32          ;
   static CString m_strTopic_bt_fl          ;
   static CString m_strTopic_bt_db          ;
   static CString m_strTopic_btSigned       ;
   static CString m_strTopic_btBigEndian    ;
   static CString m_strTopic_editChar       ;
   static CString m_strTopic_editHex8       ;
   static CString m_strTopic_editDirekt     ;
   static CString m_strTopic_editHex        ;
   static CString m_strTopic_editAs__08     ;
   static CString m_strTopic_editAs__16     ;
   static CString m_strTopic_editAs__32     ;
   static CString m_strTopic_editAs_S08     ;
   static CString m_strTopic_editAs_S16     ;
   static CString m_strTopic_editAs_S32     ;
   static CString m_strTopic_editAs__fl     ;
   static CString m_strTopic_editAs__db     ;

};
